# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from .latest import (
    LatestResource,
    AsyncLatestResource,
    LatestResourceWithRawResponse,
    AsyncLatestResourceWithRawResponse,
    LatestResourceWithStreamingResponse,
    AsyncLatestResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.magic_transit.connectors import snapshot_list_params
from .....types.magic_transit.connectors.snapshot_get_response import SnapshotGetResponse
from .....types.magic_transit.connectors.snapshot_list_response import SnapshotListResponse

__all__ = ["SnapshotsResource", "AsyncSnapshotsResource"]


class SnapshotsResource(SyncAPIResource):
    @cached_property
    def latest(self) -> LatestResource:
        return LatestResource(self._client)

    @cached_property
    def with_raw_response(self) -> SnapshotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SnapshotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SnapshotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SnapshotsResourceWithStreamingResponse(self)

    def list(
        self,
        connector_id: str,
        *,
        account_id: str,
        from_: float,
        to: float,
        cursor: str | Omit = omit,
        limit: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnapshotListResponse:
        """
        List Snapshots

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}/telemetry/snapshots",
                account_id=account_id,
                connector_id=connector_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                        "cursor": cursor,
                        "limit": limit,
                    },
                    snapshot_list_params.SnapshotListParams,
                ),
                post_parser=ResultWrapper[SnapshotListResponse]._unwrapper,
            ),
            cast_to=cast(Type[SnapshotListResponse], ResultWrapper[SnapshotListResponse]),
        )

    def get(
        self,
        snapshot_t: float,
        *,
        account_id: str,
        connector_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnapshotGetResponse:
        """
        Get Snapshot

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}/telemetry/snapshots/{snapshot_t}",
                account_id=account_id,
                connector_id=connector_id,
                snapshot_t=snapshot_t,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SnapshotGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SnapshotGetResponse], ResultWrapper[SnapshotGetResponse]),
        )


class AsyncSnapshotsResource(AsyncAPIResource):
    @cached_property
    def latest(self) -> AsyncLatestResource:
        return AsyncLatestResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSnapshotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSnapshotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSnapshotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSnapshotsResourceWithStreamingResponse(self)

    async def list(
        self,
        connector_id: str,
        *,
        account_id: str,
        from_: float,
        to: float,
        cursor: str | Omit = omit,
        limit: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnapshotListResponse:
        """
        List Snapshots

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}/telemetry/snapshots",
                account_id=account_id,
                connector_id=connector_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                        "cursor": cursor,
                        "limit": limit,
                    },
                    snapshot_list_params.SnapshotListParams,
                ),
                post_parser=ResultWrapper[SnapshotListResponse]._unwrapper,
            ),
            cast_to=cast(Type[SnapshotListResponse], ResultWrapper[SnapshotListResponse]),
        )

    async def get(
        self,
        snapshot_t: float,
        *,
        account_id: str,
        connector_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnapshotGetResponse:
        """
        Get Snapshot

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}/telemetry/snapshots/{snapshot_t}",
                account_id=account_id,
                connector_id=connector_id,
                snapshot_t=snapshot_t,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SnapshotGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SnapshotGetResponse], ResultWrapper[SnapshotGetResponse]),
        )


class SnapshotsResourceWithRawResponse:
    def __init__(self, snapshots: SnapshotsResource) -> None:
        self._snapshots = snapshots

        self.list = to_raw_response_wrapper(
            snapshots.list,
        )
        self.get = to_raw_response_wrapper(
            snapshots.get,
        )

    @cached_property
    def latest(self) -> LatestResourceWithRawResponse:
        return LatestResourceWithRawResponse(self._snapshots.latest)


class AsyncSnapshotsResourceWithRawResponse:
    def __init__(self, snapshots: AsyncSnapshotsResource) -> None:
        self._snapshots = snapshots

        self.list = async_to_raw_response_wrapper(
            snapshots.list,
        )
        self.get = async_to_raw_response_wrapper(
            snapshots.get,
        )

    @cached_property
    def latest(self) -> AsyncLatestResourceWithRawResponse:
        return AsyncLatestResourceWithRawResponse(self._snapshots.latest)


class SnapshotsResourceWithStreamingResponse:
    def __init__(self, snapshots: SnapshotsResource) -> None:
        self._snapshots = snapshots

        self.list = to_streamed_response_wrapper(
            snapshots.list,
        )
        self.get = to_streamed_response_wrapper(
            snapshots.get,
        )

    @cached_property
    def latest(self) -> LatestResourceWithStreamingResponse:
        return LatestResourceWithStreamingResponse(self._snapshots.latest)


class AsyncSnapshotsResourceWithStreamingResponse:
    def __init__(self, snapshots: AsyncSnapshotsResource) -> None:
        self._snapshots = snapshots

        self.list = async_to_streamed_response_wrapper(
            snapshots.list,
        )
        self.get = async_to_streamed_response_wrapper(
            snapshots.get,
        )

    @cached_property
    def latest(self) -> AsyncLatestResourceWithStreamingResponse:
        return AsyncLatestResourceWithStreamingResponse(self._snapshots.latest)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .previews import (
    PreviewsResource,
    AsyncPreviewsResource,
    PreviewsResourceWithRawResponse,
    AsyncPreviewsResourceWithRawResponse,
    PreviewsResourceWithStreamingResponse,
    AsyncPreviewsResourceWithStreamingResponse,
)
from .healthchecks import (
    HealthchecksResource,
    AsyncHealthchecksResource,
    HealthchecksResourceWithRawResponse,
    AsyncHealthchecksResourceWithRawResponse,
    HealthchecksResourceWithStreamingResponse,
    AsyncHealthchecksResourceWithStreamingResponse,
)

__all__ = [
    "PreviewsResource",
    "AsyncPreviewsResource",
    "PreviewsResourceWithRawResponse",
    "AsyncPreviewsResourceWithRawResponse",
    "PreviewsResourceWithStreamingResponse",
    "AsyncPreviewsResourceWithStreamingResponse",
    "HealthchecksResource",
    "AsyncHealthchecksResource",
    "HealthchecksResourceWithRawResponse",
    "AsyncHealthchecksResourceWithRawResponse",
    "HealthchecksResourceWithStreamingResponse",
    "AsyncHealthchecksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .image import Image
from ..._models import BaseModel

__all__ = ["V1ListResponse"]


class V1ListResponse(BaseModel):
    images: Optional[List[Image]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AllowPolicyListParams"]


class AllowPolicyListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    direction: Literal["asc", "desc"]
    """The sorting direction."""

    is_acceptable_sender: bool
    """
    Filter to show only policies where messages from the sender are exempted from
    Spam, Spoof, and Bulk dispositions (not Malicious or Suspicious).
    """

    is_exempt_recipient: bool
    """
    Filter to show only policies where messages to the recipient bypass all
    detections.
    """

    is_trusted_sender: bool
    """
    Filter to show only policies where messages from the sender bypass all
    detections and link following.
    """

    order: Literal["pattern", "created_at"]
    """Field to sort by."""

    page: int
    """Current page within paginated list of results."""

    pattern: str

    pattern_type: Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"]
    """
    Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
    creating or updating policies, but may be returned for existing entries.
    """

    per_page: int
    """The number of results per page. Maximum value is 1000."""

    search: str
    """Search term for filtering records. Behavior may change."""

    verify_sender: bool
    """Filter to show only policies that enforce DMARC, SPF, or DKIM authentication."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["ProviderConfigCreateResponse"]


class ProviderConfigCreateResponse(BaseModel):
    id: str

    alias: str

    default_config: bool

    gateway_id: str
    """gateway id"""

    modified_at: datetime

    provider_slug: str

    secret_id: str

    secret_preview: str

    rate_limit: Optional[float] = None

    rate_limit_period: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .domain_get_params import DomainGetParams as DomainGetParams
from .domain_get_response import DomainGetResponse as DomainGetResponse
from .internet_service_top_params import InternetServiceTopParams as InternetServiceTopParams
from .internet_service_top_response import InternetServiceTopResponse as InternetServiceTopResponse
from .internet_service_categories_params import InternetServiceCategoriesParams as InternetServiceCategoriesParams
from .internet_service_categories_response import InternetServiceCategoriesResponse as InternetServiceCategoriesResponse
from .internet_service_timeseries_groups_params import (
    InternetServiceTimeseriesGroupsParams as InternetServiceTimeseriesGroupsParams,
)
from .internet_service_timeseries_groups_response import (
    InternetServiceTimeseriesGroupsResponse as InternetServiceTimeseriesGroupsResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Type, Iterable, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncSinglePage, AsyncSinglePage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.dlp.email import rule_create_params, rule_update_params, rule_bulk_edit_params
from .....types.zero_trust.dlp.email.rule_get_response import RuleGetResponse
from .....types.zero_trust.dlp.email.rule_list_response import RuleListResponse
from .....types.zero_trust.dlp.email.rule_create_response import RuleCreateResponse
from .....types.zero_trust.dlp.email.rule_delete_response import RuleDeleteResponse
from .....types.zero_trust.dlp.email.rule_update_response import RuleUpdateResponse
from .....types.zero_trust.dlp.email.rule_bulk_edit_response import RuleBulkEditResponse

__all__ = ["RulesResource", "AsyncRulesResource"]


class RulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RulesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        action: rule_create_params.Action,
        conditions: Iterable[rule_create_params.Condition],
        enabled: bool,
        name: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleCreateResponse]:
        """
        Creates a new DLP email scanning rule that defines what content patterns to
        detect in email messages and what actions to take.

        Args:
          conditions: Triggered if all conditions match.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/dlp/email/rules", account_id=account_id),
            body=maybe_transform(
                {
                    "action": action,
                    "conditions": conditions,
                    "enabled": enabled,
                    "name": name,
                    "description": description,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleCreateResponse]], ResultWrapper[RuleCreateResponse]),
        )

    def update(
        self,
        rule_id: str,
        *,
        account_id: str,
        action: rule_update_params.Action,
        conditions: Iterable[rule_update_params.Condition],
        enabled: bool,
        name: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleUpdateResponse]:
        """
        Update email scanner rule

        Args:
          conditions: Triggered if all conditions match.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/dlp/email/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            body=maybe_transform(
                {
                    "action": action,
                    "conditions": conditions,
                    "enabled": enabled,
                    "name": name,
                    "description": description,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleUpdateResponse]], ResultWrapper[RuleUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[RuleListResponse]:
        """
        Lists all email scanner rules for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/email/rules", account_id=account_id),
            page=SyncSinglePage[RuleListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=RuleListResponse,
        )

    def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleDeleteResponse]:
        """Removes a DLP email scanning rule.

        The rule will no longer be applied to email
        messages.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/dlp/email/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleDeleteResponse]], ResultWrapper[RuleDeleteResponse]),
        )

    def bulk_edit(
        self,
        *,
        account_id: str,
        new_priorities: Dict[str, int],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleBulkEditResponse]:
        """Reorders DLP email scanning rules by updating their priority values.

        Higher
        priority rules are evaluated first.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/dlp/email/rules", account_id=account_id),
            body=maybe_transform({"new_priorities": new_priorities}, rule_bulk_edit_params.RuleBulkEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleBulkEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleBulkEditResponse]], ResultWrapper[RuleBulkEditResponse]),
        )

    def get(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleGetResponse]:
        """
        Gets detailed configuration for a specific DLP email scanning rule, including
        detection patterns and actions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/dlp/email/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleGetResponse]], ResultWrapper[RuleGetResponse]),
        )


class AsyncRulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRulesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        action: rule_create_params.Action,
        conditions: Iterable[rule_create_params.Condition],
        enabled: bool,
        name: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleCreateResponse]:
        """
        Creates a new DLP email scanning rule that defines what content patterns to
        detect in email messages and what actions to take.

        Args:
          conditions: Triggered if all conditions match.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/dlp/email/rules", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "action": action,
                    "conditions": conditions,
                    "enabled": enabled,
                    "name": name,
                    "description": description,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleCreateResponse]], ResultWrapper[RuleCreateResponse]),
        )

    async def update(
        self,
        rule_id: str,
        *,
        account_id: str,
        action: rule_update_params.Action,
        conditions: Iterable[rule_update_params.Condition],
        enabled: bool,
        name: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleUpdateResponse]:
        """
        Update email scanner rule

        Args:
          conditions: Triggered if all conditions match.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/dlp/email/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            body=await async_maybe_transform(
                {
                    "action": action,
                    "conditions": conditions,
                    "enabled": enabled,
                    "name": name,
                    "description": description,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleUpdateResponse]], ResultWrapper[RuleUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RuleListResponse, AsyncSinglePage[RuleListResponse]]:
        """
        Lists all email scanner rules for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/email/rules", account_id=account_id),
            page=AsyncSinglePage[RuleListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=RuleListResponse,
        )

    async def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleDeleteResponse]:
        """Removes a DLP email scanning rule.

        The rule will no longer be applied to email
        messages.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/dlp/email/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleDeleteResponse]], ResultWrapper[RuleDeleteResponse]),
        )

    async def bulk_edit(
        self,
        *,
        account_id: str,
        new_priorities: Dict[str, int],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleBulkEditResponse]:
        """Reorders DLP email scanning rules by updating their priority values.

        Higher
        priority rules are evaluated first.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/dlp/email/rules", account_id=account_id),
            body=await async_maybe_transform(
                {"new_priorities": new_priorities}, rule_bulk_edit_params.RuleBulkEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleBulkEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleBulkEditResponse]], ResultWrapper[RuleBulkEditResponse]),
        )

    async def get(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleGetResponse]:
        """
        Gets detailed configuration for a specific DLP email scanning rule, including
        detection patterns and actions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/dlp/email/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleGetResponse]], ResultWrapper[RuleGetResponse]),
        )


class RulesResourceWithRawResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_raw_response_wrapper(
            rules.create,
        )
        self.update = to_raw_response_wrapper(
            rules.update,
        )
        self.list = to_raw_response_wrapper(
            rules.list,
        )
        self.delete = to_raw_response_wrapper(
            rules.delete,
        )
        self.bulk_edit = to_raw_response_wrapper(
            rules.bulk_edit,
        )
        self.get = to_raw_response_wrapper(
            rules.get,
        )


class AsyncRulesResourceWithRawResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_raw_response_wrapper(
            rules.create,
        )
        self.update = async_to_raw_response_wrapper(
            rules.update,
        )
        self.list = async_to_raw_response_wrapper(
            rules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            rules.delete,
        )
        self.bulk_edit = async_to_raw_response_wrapper(
            rules.bulk_edit,
        )
        self.get = async_to_raw_response_wrapper(
            rules.get,
        )


class RulesResourceWithStreamingResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_streamed_response_wrapper(
            rules.create,
        )
        self.update = to_streamed_response_wrapper(
            rules.update,
        )
        self.list = to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = to_streamed_response_wrapper(
            rules.delete,
        )
        self.bulk_edit = to_streamed_response_wrapper(
            rules.bulk_edit,
        )
        self.get = to_streamed_response_wrapper(
            rules.get,
        )


class AsyncRulesResourceWithStreamingResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_streamed_response_wrapper(
            rules.create,
        )
        self.update = async_to_streamed_response_wrapper(
            rules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            rules.delete,
        )
        self.bulk_edit = async_to_streamed_response_wrapper(
            rules.bulk_edit,
        )
        self.get = async_to_streamed_response_wrapper(
            rules.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ServiceTokenCreateParams"]


class ServiceTokenCreateParams(TypedDict, total=False):
    name: Required[str]
    """The name of the service token."""

    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    client_secret_version: float
    """
    A version number identifying the current `client_secret` associated with the
    service token. Incrementing it triggers a rotation; the previous secret will
    still be accepted until the time indicated by
    `previous_client_secret_expires_at`.
    """

    duration: str
    """The duration for how long the service token will be valid.

    Must be in the format `300ms` or `2h45m`. Valid time units are: ns, us (or µs),
    ms, s, m, h. The default is 1 year in hours (8760h).
    """

    previous_client_secret_expires_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The expiration of the previous `client_secret`.

    This can be modified at any point after a rotation. For example, you may extend
    it further into the future if you need more time to update services with the new
    secret; or move it into the past to immediately invalidate the previous token in
    case of compromise.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["KeyBulkUpdateResponse"]


class KeyBulkUpdateResponse(BaseModel):
    successful_key_count: Optional[float] = None
    """Number of keys successfully updated."""

    unsuccessful_keys: Optional[List[str]] = None
    """Name of the keys that failed to be fully updated. They should be retried."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from .applications import (
    ApplicationsResource,
    AsyncApplicationsResource,
    ApplicationsResourceWithRawResponse,
    AsyncApplicationsResourceWithRawResponse,
    ApplicationsResourceWithStreamingResponse,
    AsyncApplicationsResourceWithStreamingResponse,
)

__all__ = ["ResourceLibraryResource", "AsyncResourceLibraryResource"]


class ResourceLibraryResource(SyncAPIResource):
    @cached_property
    def applications(self) -> ApplicationsResource:
        return ApplicationsResource(self._client)

    @cached_property
    def categories(self) -> CategoriesResource:
        return CategoriesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ResourceLibraryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ResourceLibraryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResourceLibraryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ResourceLibraryResourceWithStreamingResponse(self)


class AsyncResourceLibraryResource(AsyncAPIResource):
    @cached_property
    def applications(self) -> AsyncApplicationsResource:
        return AsyncApplicationsResource(self._client)

    @cached_property
    def categories(self) -> AsyncCategoriesResource:
        return AsyncCategoriesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncResourceLibraryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncResourceLibraryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResourceLibraryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncResourceLibraryResourceWithStreamingResponse(self)


class ResourceLibraryResourceWithRawResponse:
    def __init__(self, resource_library: ResourceLibraryResource) -> None:
        self._resource_library = resource_library

    @cached_property
    def applications(self) -> ApplicationsResourceWithRawResponse:
        return ApplicationsResourceWithRawResponse(self._resource_library.applications)

    @cached_property
    def categories(self) -> CategoriesResourceWithRawResponse:
        return CategoriesResourceWithRawResponse(self._resource_library.categories)


class AsyncResourceLibraryResourceWithRawResponse:
    def __init__(self, resource_library: AsyncResourceLibraryResource) -> None:
        self._resource_library = resource_library

    @cached_property
    def applications(self) -> AsyncApplicationsResourceWithRawResponse:
        return AsyncApplicationsResourceWithRawResponse(self._resource_library.applications)

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithRawResponse:
        return AsyncCategoriesResourceWithRawResponse(self._resource_library.categories)


class ResourceLibraryResourceWithStreamingResponse:
    def __init__(self, resource_library: ResourceLibraryResource) -> None:
        self._resource_library = resource_library

    @cached_property
    def applications(self) -> ApplicationsResourceWithStreamingResponse:
        return ApplicationsResourceWithStreamingResponse(self._resource_library.applications)

    @cached_property
    def categories(self) -> CategoriesResourceWithStreamingResponse:
        return CategoriesResourceWithStreamingResponse(self._resource_library.categories)


class AsyncResourceLibraryResourceWithStreamingResponse:
    def __init__(self, resource_library: AsyncResourceLibraryResource) -> None:
        self._resource_library = resource_library

    @cached_property
    def applications(self) -> AsyncApplicationsResourceWithStreamingResponse:
        return AsyncApplicationsResourceWithStreamingResponse(self._resource_library.applications)

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithStreamingResponse:
        return AsyncCategoriesResourceWithStreamingResponse(self._resource_library.categories)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["IdentityProviderListParams"]


class IdentityProviderListParams(TypedDict, total=False):
    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    page: int
    """Page number of results."""

    per_page: int
    """Number of results per page."""

    scim_enabled: str
    """
    Indicates to Access to only retrieve identity providers that have the System for
    Cross-Domain Identity Management (SCIM) enabled.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import Body, Query, Headers, NotGiven, not_given
from ....._utils import path_template
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncSinglePage, AsyncSinglePage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.access.users.failed_login_list_response import FailedLoginListResponse

__all__ = ["FailedLoginsResource", "AsyncFailedLoginsResource"]


class FailedLoginsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FailedLoginsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FailedLoginsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FailedLoginsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FailedLoginsResourceWithStreamingResponse(self)

    def list(
        self,
        user_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[FailedLoginListResponse]:
        """
        Get all failed login attempts for a single user.

        Args:
          account_id: Identifier.

          user_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/access/users/{user_id}/failed_logins", account_id=account_id, user_id=user_id
            ),
            page=SyncSinglePage[FailedLoginListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FailedLoginListResponse,
        )


class AsyncFailedLoginsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFailedLoginsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFailedLoginsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFailedLoginsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFailedLoginsResourceWithStreamingResponse(self)

    def list(
        self,
        user_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FailedLoginListResponse, AsyncSinglePage[FailedLoginListResponse]]:
        """
        Get all failed login attempts for a single user.

        Args:
          account_id: Identifier.

          user_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/access/users/{user_id}/failed_logins", account_id=account_id, user_id=user_id
            ),
            page=AsyncSinglePage[FailedLoginListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FailedLoginListResponse,
        )


class FailedLoginsResourceWithRawResponse:
    def __init__(self, failed_logins: FailedLoginsResource) -> None:
        self._failed_logins = failed_logins

        self.list = to_raw_response_wrapper(
            failed_logins.list,
        )


class AsyncFailedLoginsResourceWithRawResponse:
    def __init__(self, failed_logins: AsyncFailedLoginsResource) -> None:
        self._failed_logins = failed_logins

        self.list = async_to_raw_response_wrapper(
            failed_logins.list,
        )


class FailedLoginsResourceWithStreamingResponse:
    def __init__(self, failed_logins: FailedLoginsResource) -> None:
        self._failed_logins = failed_logins

        self.list = to_streamed_response_wrapper(
            failed_logins.list,
        )


class AsyncFailedLoginsResourceWithStreamingResponse:
    def __init__(self, failed_logins: AsyncFailedLoginsResource) -> None:
        self._failed_logins = failed_logins

        self.list = async_to_streamed_response_wrapper(
            failed_logins.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["DomainEditParams"]


class DomainEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    allowed_delivery_modes: List[Literal["DIRECT", "BCC", "JOURNAL", "API", "RETRO_SCAN"]]

    domain: str

    drop_dispositions: List[
        Literal[
            "MALICIOUS",
            "MALICIOUS-BEC",
            "SUSPICIOUS",
            "SPOOF",
            "SPAM",
            "BULK",
            "ENCRYPTED",
            "EXTERNAL",
            "UNKNOWN",
            "NONE",
        ]
    ]

    folder: Literal["AllItems", "Inbox"]

    integration_id: Optional[str]

    ip_restrictions: SequenceNotStr[str]

    lookback_hops: int

    regions: List[Literal["GLOBAL", "AU", "DE", "IN", "US"]]

    require_tls_inbound: bool

    require_tls_outbound: bool

    transport: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .apps import (
    AppsResource,
    AsyncAppsResource,
    AppsResourceWithRawResponse,
    AsyncAppsResourceWithRawResponse,
    AppsResourceWithStreamingResponse,
    AsyncAppsResourceWithStreamingResponse,
)
from .spectrum import (
    SpectrumResource,
    AsyncSpectrumResource,
    SpectrumResourceWithRawResponse,
    AsyncSpectrumResourceWithRawResponse,
    SpectrumResourceWithStreamingResponse,
    AsyncSpectrumResourceWithStreamingResponse,
)
from .analytics import (
    AnalyticsResource,
    AsyncAnalyticsResource,
    AnalyticsResourceWithRawResponse,
    AsyncAnalyticsResourceWithRawResponse,
    AnalyticsResourceWithStreamingResponse,
    AsyncAnalyticsResourceWithStreamingResponse,
)

__all__ = [
    "AnalyticsResource",
    "AsyncAnalyticsResource",
    "AnalyticsResourceWithRawResponse",
    "AsyncAnalyticsResourceWithRawResponse",
    "AnalyticsResourceWithStreamingResponse",
    "AsyncAnalyticsResourceWithStreamingResponse",
    "AppsResource",
    "AsyncAppsResource",
    "AppsResourceWithRawResponse",
    "AsyncAppsResourceWithRawResponse",
    "AppsResourceWithStreamingResponse",
    "AsyncAppsResourceWithStreamingResponse",
    "SpectrumResource",
    "AsyncSpectrumResource",
    "SpectrumResourceWithRawResponse",
    "AsyncSpectrumResourceWithRawResponse",
    "SpectrumResourceWithStreamingResponse",
    "AsyncSpectrumResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .app_post_params import AppPostParams as AppPostParams
from .app_get_response import AppGetResponse as AppGetResponse
from .app_post_response import AppPostResponse as AppPostResponse
from .preset_get_params import PresetGetParams as PresetGetParams
from .meeting_get_params import MeetingGetParams as MeetingGetParams
from .preset_get_response import PresetGetResponse as PresetGetResponse
from .meeting_get_response import MeetingGetResponse as MeetingGetResponse
from .preset_create_params import PresetCreateParams as PresetCreateParams
from .preset_update_params import PresetUpdateParams as PresetUpdateParams
from .meeting_create_params import MeetingCreateParams as MeetingCreateParams
from .preset_create_response import PresetCreateResponse as PresetCreateResponse
from .preset_delete_response import PresetDeleteResponse as PresetDeleteResponse
from .preset_update_response import PresetUpdateResponse as PresetUpdateResponse
from .meeting_create_response import MeetingCreateResponse as MeetingCreateResponse
from .session_get_sessions_params import SessionGetSessionsParams as SessionGetSessionsParams
from .webhook_edit_webhook_params import WebhookEditWebhookParams as WebhookEditWebhookParams
from .session_get_sessions_response import SessionGetSessionsResponse as SessionGetSessionsResponse
from .webhook_create_webhook_params import WebhookCreateWebhookParams as WebhookCreateWebhookParams
from .webhook_edit_webhook_response import WebhookEditWebhookResponse as WebhookEditWebhookResponse
from .webhook_get_webhooks_response import WebhookGetWebhooksResponse as WebhookGetWebhooksResponse
from .meeting_add_participant_params import MeetingAddParticipantParams as MeetingAddParticipantParams
from .webhook_replace_webhook_params import WebhookReplaceWebhookParams as WebhookReplaceWebhookParams
from .meeting_edit_participant_params import MeetingEditParticipantParams as MeetingEditParticipantParams
from .recording_get_recordings_params import RecordingGetRecordingsParams as RecordingGetRecordingsParams
from .webhook_create_webhook_response import WebhookCreateWebhookResponse as WebhookCreateWebhookResponse
from .webhook_delete_webhook_response import WebhookDeleteWebhookResponse as WebhookDeleteWebhookResponse
from .meeting_add_participant_response import MeetingAddParticipantResponse as MeetingAddParticipantResponse
from .meeting_get_meeting_by_id_params import MeetingGetMeetingByIDParams as MeetingGetMeetingByIDParams
from .preset_get_preset_by_id_response import PresetGetPresetByIDResponse as PresetGetPresetByIDResponse
from .webhook_replace_webhook_response import WebhookReplaceWebhookResponse as WebhookReplaceWebhookResponse
from .active_session_create_poll_params import ActiveSessionCreatePollParams as ActiveSessionCreatePollParams
from .meeting_edit_participant_response import MeetingEditParticipantResponse as MeetingEditParticipantResponse
from .recording_get_recordings_response import RecordingGetRecordingsResponse as RecordingGetRecordingsResponse
from .recording_start_recordings_params import RecordingStartRecordingsParams as RecordingStartRecordingsParams
from .session_get_session_chat_response import SessionGetSessionChatResponse as SessionGetSessionChatResponse
from .analytics_get_org_analytics_params import AnalyticsGetOrgAnalyticsParams as AnalyticsGetOrgAnalyticsParams
from .meeting_get_meeting_by_id_response import MeetingGetMeetingByIDResponse as MeetingGetMeetingByIDResponse
from .session_get_session_details_params import SessionGetSessionDetailsParams as SessionGetSessionDetailsParams
from .webhook_get_webhook_by_id_response import WebhookGetWebhookByIDResponse as WebhookGetWebhookByIDResponse
from .active_session_create_poll_response import ActiveSessionCreatePollResponse as ActiveSessionCreatePollResponse
from .livestream_get_org_analytics_params import LivestreamGetOrgAnalyticsParams as LivestreamGetOrgAnalyticsParams
from .meeting_update_meeting_by_id_params import MeetingUpdateMeetingByIDParams as MeetingUpdateMeetingByIDParams
from .recording_start_recordings_response import RecordingStartRecordingsResponse as RecordingStartRecordingsResponse
from .analytics_get_org_analytics_response import AnalyticsGetOrgAnalyticsResponse as AnalyticsGetOrgAnalyticsResponse
from .meeting_replace_meeting_by_id_params import MeetingReplaceMeetingByIDParams as MeetingReplaceMeetingByIDParams
from .recording_get_one_recording_response import RecordingGetOneRecordingResponse as RecordingGetOneRecordingResponse
from .session_get_session_details_response import SessionGetSessionDetailsResponse as SessionGetSessionDetailsResponse
from .session_get_session_summary_response import SessionGetSessionSummaryResponse as SessionGetSessionSummaryResponse
from .livestream_get_all_livestreams_params import (
    LivestreamGetAllLivestreamsParams as LivestreamGetAllLivestreamsParams,
)
from .livestream_get_org_analytics_response import (
    LivestreamGetOrgAnalyticsResponse as LivestreamGetOrgAnalyticsResponse,
)
from .meeting_update_meeting_by_id_response import MeetingUpdateMeetingByIDResponse as MeetingUpdateMeetingByIDResponse
from .meeting_replace_meeting_by_id_response import (
    MeetingReplaceMeetingByIDResponse as MeetingReplaceMeetingByIDResponse,
)
from .recording_start_track_recording_params import (
    RecordingStartTrackRecordingParams as RecordingStartTrackRecordingParams,
)
from .active_session_kick_participants_params import (
    ActiveSessionKickParticipantsParams as ActiveSessionKickParticipantsParams,
)
from .livestream_get_all_livestreams_response import (
    LivestreamGetAllLivestreamsResponse as LivestreamGetAllLivestreamsResponse,
)
from .meeting_get_meeting_participants_params import (
    MeetingGetMeetingParticipantsParams as MeetingGetMeetingParticipantsParams,
)
from .session_get_session_participants_params import (
    SessionGetSessionParticipantsParams as SessionGetSessionParticipantsParams,
)
from .meeting_get_meeting_participant_response import (
    MeetingGetMeetingParticipantResponse as MeetingGetMeetingParticipantResponse,
)
from .recording_get_active_recordings_response import (
    RecordingGetActiveRecordingsResponse as RecordingGetActiveRecordingsResponse,
)
from .session_get_session_transcripts_response import (
    SessionGetSessionTranscriptsResponse as SessionGetSessionTranscriptsResponse,
)
from .active_session_kick_participants_response import (
    ActiveSessionKickParticipantsResponse as ActiveSessionKickParticipantsResponse,
)
from .meeting_get_meeting_participants_response import (
    MeetingGetMeetingParticipantsResponse as MeetingGetMeetingParticipantsResponse,
)
from .session_get_session_participants_response import (
    SessionGetSessionParticipantsResponse as SessionGetSessionParticipantsResponse,
)
from .active_session_get_active_session_response import (
    ActiveSessionGetActiveSessionResponse as ActiveSessionGetActiveSessionResponse,
)
from .meeting_refresh_participant_token_response import (
    MeetingRefreshParticipantTokenResponse as MeetingRefreshParticipantTokenResponse,
)
from .meeting_delete_meeting_participant_response import (
    MeetingDeleteMeetingParticipantResponse as MeetingDeleteMeetingParticipantResponse,
)
from .recording_pause_resume_stop_recording_params import (
    RecordingPauseResumeStopRecordingParams as RecordingPauseResumeStopRecordingParams,
)
from .active_session_kick_all_participants_response import (
    ActiveSessionKickAllParticipantsResponse as ActiveSessionKickAllParticipantsResponse,
)
from .recording_pause_resume_stop_recording_response import (
    RecordingPauseResumeStopRecordingResponse as RecordingPauseResumeStopRecordingResponse,
)
from .session_get_session_participant_details_params import (
    SessionGetSessionParticipantDetailsParams as SessionGetSessionParticipantDetailsParams,
)
from .livestream_create_independent_livestream_params import (
    LivestreamCreateIndependentLivestreamParams as LivestreamCreateIndependentLivestreamParams,
)
from .livestream_start_livestreaming_a_meeting_params import (
    LivestreamStartLivestreamingAMeetingParams as LivestreamStartLivestreamingAMeetingParams,
)
from .livestream_stop_livestreaming_a_meeting_response import (
    LivestreamStopLivestreamingAMeetingResponse as LivestreamStopLivestreamingAMeetingResponse,
)
from .session_get_participant_data_from_peer_id_params import (
    SessionGetParticipantDataFromPeerIDParams as SessionGetParticipantDataFromPeerIDParams,
)
from .session_get_session_participant_details_response import (
    SessionGetSessionParticipantDetailsResponse as SessionGetSessionParticipantDetailsResponse,
)
from .livestream_create_independent_livestream_response import (
    LivestreamCreateIndependentLivestreamResponse as LivestreamCreateIndependentLivestreamResponse,
)
from .livestream_start_livestreaming_a_meeting_response import (
    LivestreamStartLivestreamingAMeetingResponse as LivestreamStartLivestreamingAMeetingResponse,
)
from .livestream_get_meeting_active_livestreams_response import (
    LivestreamGetMeetingActiveLivestreamsResponse as LivestreamGetMeetingActiveLivestreamsResponse,
)
from .session_get_participant_data_from_peer_id_response import (
    SessionGetParticipantDataFromPeerIDResponse as SessionGetParticipantDataFromPeerIDResponse,
)
from .livestream_get_livestream_analytics_complete_params import (
    LivestreamGetLivestreamAnalyticsCompleteParams as LivestreamGetLivestreamAnalyticsCompleteParams,
)
from .livestream_get_livestream_analytics_complete_response import (
    LivestreamGetLivestreamAnalyticsCompleteResponse as LivestreamGetLivestreamAnalyticsCompleteResponse,
)
from .livestream_get_livestream_session_for_livestream_id_params import (
    LivestreamGetLivestreamSessionForLivestreamIDParams as LivestreamGetLivestreamSessionForLivestreamIDParams,
)
from .livestream_get_active_livestreams_for_livestream_id_response import (
    LivestreamGetActiveLivestreamsForLivestreamIDResponse as LivestreamGetActiveLivestreamsForLivestreamIDResponse,
)
from .livestream_get_livestream_session_for_livestream_id_response import (
    LivestreamGetLivestreamSessionForLivestreamIDResponse as LivestreamGetLivestreamSessionForLivestreamIDResponse,
)
from .livestream_get_livestream_session_details_for_session_id_response import (
    LivestreamGetLivestreamSessionDetailsForSessionIDResponse as LivestreamGetLivestreamSessionDetailsForSessionIDResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...types.user import organization_list_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.user.organization import Organization
from ...types.user.organization_delete_response import OrganizationDeleteResponse

__all__ = ["OrganizationsResource", "AsyncOrganizationsResource"]


class OrganizationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OrganizationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OrganizationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OrganizationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OrganizationsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        name: str | Omit = omit,
        order: Literal["id", "name", "status"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["member", "invited"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Organization]:
        """
        Lists organizations the user is associated with.

        Args:
          direction: Direction to order organizations.

          match: Whether to match all search requirements or at least one (any).

          name: Organization name.

          order: Field to order organizations by.

          page: Page number of paginated results.

          per_page: Number of organizations per page.

          status: Whether the user is a member of the organization or has an inivitation pending.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/user/organizations",
            page=SyncV4PagePaginationArray[Organization],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "name": name,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    organization_list_params.OrganizationListParams,
                ),
            ),
            model=Organization,
        )

    @typing_extensions.deprecated("deprecated")
    def delete(
        self,
        organization_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationDeleteResponse:
        """
        Removes association to an organization.

        Args:
          organization_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not organization_id:
            raise ValueError(f"Expected a non-empty value for `organization_id` but received {organization_id!r}")
        return self._delete(
            path_template("/user/organizations/{organization_id}", organization_id=organization_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationDeleteResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def get(
        self,
        organization_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Gets a specific organization the user is associated with.

        Args:
          organization_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not organization_id:
            raise ValueError(f"Expected a non-empty value for `organization_id` but received {organization_id!r}")
        return self._get(
            path_template("/user/organizations/{organization_id}", organization_id=organization_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncOrganizationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOrganizationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOrganizationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOrganizationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOrganizationsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        name: str | Omit = omit,
        order: Literal["id", "name", "status"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["member", "invited"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Organization, AsyncV4PagePaginationArray[Organization]]:
        """
        Lists organizations the user is associated with.

        Args:
          direction: Direction to order organizations.

          match: Whether to match all search requirements or at least one (any).

          name: Organization name.

          order: Field to order organizations by.

          page: Page number of paginated results.

          per_page: Number of organizations per page.

          status: Whether the user is a member of the organization or has an inivitation pending.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/user/organizations",
            page=AsyncV4PagePaginationArray[Organization],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "name": name,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    organization_list_params.OrganizationListParams,
                ),
            ),
            model=Organization,
        )

    @typing_extensions.deprecated("deprecated")
    async def delete(
        self,
        organization_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationDeleteResponse:
        """
        Removes association to an organization.

        Args:
          organization_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not organization_id:
            raise ValueError(f"Expected a non-empty value for `organization_id` but received {organization_id!r}")
        return await self._delete(
            path_template("/user/organizations/{organization_id}", organization_id=organization_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationDeleteResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def get(
        self,
        organization_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Gets a specific organization the user is associated with.

        Args:
          organization_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not organization_id:
            raise ValueError(f"Expected a non-empty value for `organization_id` but received {organization_id!r}")
        return await self._get(
            path_template("/user/organizations/{organization_id}", organization_id=organization_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class OrganizationsResourceWithRawResponse:
    def __init__(self, organizations: OrganizationsResource) -> None:
        self._organizations = organizations

        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                organizations.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                organizations.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                organizations.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncOrganizationsResourceWithRawResponse:
    def __init__(self, organizations: AsyncOrganizationsResource) -> None:
        self._organizations = organizations

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                organizations.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                organizations.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                organizations.get,  # pyright: ignore[reportDeprecated],
            )
        )


class OrganizationsResourceWithStreamingResponse:
    def __init__(self, organizations: OrganizationsResource) -> None:
        self._organizations = organizations

        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                organizations.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                organizations.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                organizations.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncOrganizationsResourceWithStreamingResponse:
    def __init__(self, organizations: AsyncOrganizationsResource) -> None:
        self._organizations = organizations

        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                organizations.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                organizations.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                organizations.get,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.workers import subdomain_update_params
from ...types.workers.subdomain_get_response import SubdomainGetResponse
from ...types.workers.subdomain_update_response import SubdomainUpdateResponse

__all__ = ["SubdomainsResource", "AsyncSubdomainsResource"]


class SubdomainsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubdomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SubdomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubdomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SubdomainsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        subdomain: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainUpdateResponse:
        """
        Creates a Workers subdomain for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/workers/subdomain", account_id=account_id),
            body=maybe_transform({"subdomain": subdomain}, subdomain_update_params.SubdomainUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainUpdateResponse], ResultWrapper[SubdomainUpdateResponse]),
        )

    def delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a Workers subdomain for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/accounts/{account_id}/workers/subdomain", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainGetResponse:
        """
        Returns a Workers subdomain for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/workers/subdomain", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainGetResponse], ResultWrapper[SubdomainGetResponse]),
        )


class AsyncSubdomainsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubdomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubdomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubdomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSubdomainsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        subdomain: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainUpdateResponse:
        """
        Creates a Workers subdomain for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/workers/subdomain", account_id=account_id),
            body=await async_maybe_transform({"subdomain": subdomain}, subdomain_update_params.SubdomainUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainUpdateResponse], ResultWrapper[SubdomainUpdateResponse]),
        )

    async def delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a Workers subdomain for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/accounts/{account_id}/workers/subdomain", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainGetResponse:
        """
        Returns a Workers subdomain for an account.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/workers/subdomain", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainGetResponse], ResultWrapper[SubdomainGetResponse]),
        )


class SubdomainsResourceWithRawResponse:
    def __init__(self, subdomains: SubdomainsResource) -> None:
        self._subdomains = subdomains

        self.update = to_raw_response_wrapper(
            subdomains.update,
        )
        self.delete = to_raw_response_wrapper(
            subdomains.delete,
        )
        self.get = to_raw_response_wrapper(
            subdomains.get,
        )


class AsyncSubdomainsResourceWithRawResponse:
    def __init__(self, subdomains: AsyncSubdomainsResource) -> None:
        self._subdomains = subdomains

        self.update = async_to_raw_response_wrapper(
            subdomains.update,
        )
        self.delete = async_to_raw_response_wrapper(
            subdomains.delete,
        )
        self.get = async_to_raw_response_wrapper(
            subdomains.get,
        )


class SubdomainsResourceWithStreamingResponse:
    def __init__(self, subdomains: SubdomainsResource) -> None:
        self._subdomains = subdomains

        self.update = to_streamed_response_wrapper(
            subdomains.update,
        )
        self.delete = to_streamed_response_wrapper(
            subdomains.delete,
        )
        self.get = to_streamed_response_wrapper(
            subdomains.get,
        )


class AsyncSubdomainsResourceWithStreamingResponse:
    def __init__(self, subdomains: AsyncSubdomainsResource) -> None:
        self._subdomains = subdomains

        self.update = async_to_streamed_response_wrapper(
            subdomains.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            subdomains.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            subdomains.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr
from .dcv_method import DCVMethod
from .bundle_method import BundleMethod
from ..shared.certificate_ca import CertificateCA
from .domain_validation_type import DomainValidationType

__all__ = ["CustomHostnameEditParams", "SSL", "SSLCustomCERTBundle", "SSLSettings"]


class CustomHostnameEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    custom_metadata: Dict[str, str]
    """Unique key/value metadata for this hostname.

    These are per-hostname (customer) settings.
    """

    custom_origin_server: str
    """
    a valid hostname that’s been added to your DNS zone as an A, AAAA, or CNAME
    record.
    """

    custom_origin_sni: str
    """
    A hostname that will be sent to your custom origin server as SNI for TLS
    handshake. This can be a valid subdomain of the zone or custom origin server
    name or the string ':request_host_header:' which will cause the host header in
    the request to be used as SNI. Not configurable with default/fallback origin
    server.
    """

    ssl: SSL
    """SSL properties used when creating the custom hostname."""


class SSLCustomCERTBundle(TypedDict, total=False):
    custom_certificate: Required[str]
    """If a custom uploaded certificate is used."""

    custom_key: Required[str]
    """The key for a custom uploaded certificate."""


class SSLSettings(TypedDict, total=False):
    """SSL specific settings."""

    ciphers: SequenceNotStr[str]
    """An allowlist of ciphers for TLS termination.

    These ciphers must be in the BoringSSL format.
    """

    early_hints: Literal["on", "off"]
    """Whether or not Early Hints is enabled."""

    http2: Literal["on", "off"]
    """Whether or not HTTP2 is enabled."""

    min_tls_version: Literal["1.0", "1.1", "1.2", "1.3"]
    """The minimum TLS version supported."""

    tls_1_3: Literal["on", "off"]
    """Whether or not TLS 1.3 is enabled."""


class SSL(TypedDict, total=False):
    """SSL properties used when creating the custom hostname."""

    bundle_method: BundleMethod
    """
    A ubiquitous bundle has the highest probability of being verified everywhere,
    even by clients using outdated or unusual trust stores. An optimal bundle uses
    the shortest chain and newest intermediates. And the force bundle verifies the
    chain, but does not otherwise modify it.
    """

    certificate_authority: CertificateCA
    """The Certificate Authority that will issue the certificate"""

    cloudflare_branding: bool
    """Whether or not to add Cloudflare Branding for the order.

    This will add a subdomain of sni.cloudflaressl.com as the Common Name if set to
    true
    """

    custom_cert_bundle: Iterable[SSLCustomCERTBundle]
    """Array of custom certificate and key pairs (1 or 2 pairs allowed)"""

    custom_certificate: str
    """If a custom uploaded certificate is used."""

    custom_csr_id: str
    """The identifier for the Custom CSR that was used."""

    custom_key: str
    """The key for a custom uploaded certificate."""

    method: DCVMethod
    """Domain control validation (DCV) method used for this hostname."""

    settings: SSLSettings
    """SSL specific settings."""

    type: DomainValidationType
    """Level of validation to be used for this hostname.

    Domain validation (dv) must be used.
    """

    wildcard: bool
    """Indicates whether the certificate covers a wildcard."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.brand_protection.v2 import logo_get_params, logo_create_params
from ....types.brand_protection.v2.logo_get_response import LogoGetResponse
from ....types.brand_protection.v2.logo_create_response import LogoCreateResponse
from ....types.brand_protection.v2.logo_delete_response import LogoDeleteResponse

__all__ = ["LogosResource", "AsyncLogosResource"]


class LogosResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LogosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LogosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LogosResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        image_data: str,
        similarity_threshold: float,
        tag: str,
        search_lookback: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoCreateResponse:
        """
        Create a new saved brand protection logo query for visual similarity matching

        Args:
          image_data: Base64 encoded image data. Can include data URI prefix (e.g.,
              'data:image/png;base64,...') or just the base64 string.

          similarity_threshold: Minimum similarity score (0-1) required for visual matches

          tag: Unique identifier for the logo query

          search_lookback: If true, search historic scanned images for matches above the similarity
              threshold

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/cloudforce-one/v2/brand-protection/logo/queries", account_id=account_id
            ),
            body=maybe_transform(
                {
                    "image_data": image_data,
                    "similarity_threshold": similarity_threshold,
                    "tag": tag,
                    "search_lookback": search_lookback,
                },
                logo_create_params.LogoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LogoCreateResponse,
        )

    def delete(
        self,
        query_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoDeleteResponse:
        """Delete a saved brand protection logo query.

        Returns 404 if the query ID doesn't
        exist.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not query_id:
            raise ValueError(f"Expected a non-empty value for `query_id` but received {query_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/cloudforce-one/v2/brand-protection/logo/queries/{query_id}",
                account_id=account_id,
                query_id=query_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LogoDeleteResponse,
        )

    def get(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        download: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoGetResponse:
        """Get all saved brand protection logo queries for an account.

        Optionally specify
        id to get a single query. Set download=true to include base64-encoded image
        data.

        Args:
          id: Optional query ID to retrieve a specific logo query

          download: If true, include base64-encoded image data in the response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/v2/brand-protection/logo/queries", account_id=account_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "download": download,
                    },
                    logo_get_params.LogoGetParams,
                ),
            ),
            cast_to=LogoGetResponse,
        )


class AsyncLogosResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLogosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLogosResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        image_data: str,
        similarity_threshold: float,
        tag: str,
        search_lookback: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoCreateResponse:
        """
        Create a new saved brand protection logo query for visual similarity matching

        Args:
          image_data: Base64 encoded image data. Can include data URI prefix (e.g.,
              'data:image/png;base64,...') or just the base64 string.

          similarity_threshold: Minimum similarity score (0-1) required for visual matches

          tag: Unique identifier for the logo query

          search_lookback: If true, search historic scanned images for matches above the similarity
              threshold

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/cloudforce-one/v2/brand-protection/logo/queries", account_id=account_id
            ),
            body=await async_maybe_transform(
                {
                    "image_data": image_data,
                    "similarity_threshold": similarity_threshold,
                    "tag": tag,
                    "search_lookback": search_lookback,
                },
                logo_create_params.LogoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LogoCreateResponse,
        )

    async def delete(
        self,
        query_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoDeleteResponse:
        """Delete a saved brand protection logo query.

        Returns 404 if the query ID doesn't
        exist.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not query_id:
            raise ValueError(f"Expected a non-empty value for `query_id` but received {query_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/cloudforce-one/v2/brand-protection/logo/queries/{query_id}",
                account_id=account_id,
                query_id=query_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LogoDeleteResponse,
        )

    async def get(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        download: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoGetResponse:
        """Get all saved brand protection logo queries for an account.

        Optionally specify
        id to get a single query. Set download=true to include base64-encoded image
        data.

        Args:
          id: Optional query ID to retrieve a specific logo query

          download: If true, include base64-encoded image data in the response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/cloudforce-one/v2/brand-protection/logo/queries", account_id=account_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "download": download,
                    },
                    logo_get_params.LogoGetParams,
                ),
            ),
            cast_to=LogoGetResponse,
        )


class LogosResourceWithRawResponse:
    def __init__(self, logos: LogosResource) -> None:
        self._logos = logos

        self.create = to_raw_response_wrapper(
            logos.create,
        )
        self.delete = to_raw_response_wrapper(
            logos.delete,
        )
        self.get = to_raw_response_wrapper(
            logos.get,
        )


class AsyncLogosResourceWithRawResponse:
    def __init__(self, logos: AsyncLogosResource) -> None:
        self._logos = logos

        self.create = async_to_raw_response_wrapper(
            logos.create,
        )
        self.delete = async_to_raw_response_wrapper(
            logos.delete,
        )
        self.get = async_to_raw_response_wrapper(
            logos.get,
        )


class LogosResourceWithStreamingResponse:
    def __init__(self, logos: LogosResource) -> None:
        self._logos = logos

        self.create = to_streamed_response_wrapper(
            logos.create,
        )
        self.delete = to_streamed_response_wrapper(
            logos.delete,
        )
        self.get = to_streamed_response_wrapper(
            logos.get,
        )


class AsyncLogosResourceWithStreamingResponse:
    def __init__(self, logos: AsyncLogosResource) -> None:
        self._logos = logos

        self.create = async_to_streamed_response_wrapper(
            logos.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            logos.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            logos.get,
        )
